#include <autre.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

extern int color_rouge, color_vert, color_bleu;

int get_list_pos(Point *p, Point **pv, int listlen)
{
  int count;

  for(count=0;count<listlen;count++) 
    if(pv[count]==p) return count;
  return 0;
}


char *get_line_from_file(FILE *fp)
{
  char *result, c;
  GString *string;

  string=g_string_sized_new(0);
  c=fgetc(fp);

  if((c=='\n') || (c==EOF)) {
    result=(char *)malloc(2);
    result[0]='\0';
    return result;
  }

  while((c!=EOF)&&(c!='\n')) {
    g_string_append_c(string, c);
    c=fgetc(fp);
  }
  
  result=(char *)malloc(string->len+1);
  strncpy(result, string->str, string->len);
  
  result[string->len]='\0';

  g_string_free(string, 1);
      
  return result;
}


void model_save_a(Model *m, char *filename)
{
  GList *c;
  Tri *tri;
  Point *p, **pv;
  int count=0, listlen;
  FILE *fp;

  fp=fopen(filename, "w");
  if(fp==NULL) return;

  listlen=g_list_length(m->points);
  
  pv=(Point **)malloc(sizeof(Point *)*listlen);

  fprintf(fp, "Visu3D ASCII\n");

  for(c=m->points;c;c=c->next, count++) {
    p=(Point *)c->data;
    fprintf(fp, "POINT %d\n", count);
    fprintf(fp, " %f\n %f\n %f\n", p->v[0]-50, p->v[1]-50, p->v[2]-50);
    pv[count]=p;
  }
  
  count=0;
  for(c=m->triangles;c;c=c->next, count++) {
    tri=(Tri *)c->data; 
    fprintf(fp, "TRIANGLE %d\n", count);
    fprintf(fp, " POINTS\n");
    fprintf(fp, "  %d\n  %d\n  %d\n", 
	   get_list_pos(tri->verticies[0], pv, listlen),
	   get_list_pos(tri->verticies[1], pv, listlen),
	   get_list_pos(tri->verticies[2], pv, listlen));
    fprintf(fp, " COULEURS\n");
    fprintf(fp, "  %d\n  %d\n  %d\n",
	    tri->rouge, 
	    tri->vert, 
	    tri->bleu);

  }
  fprintf(fp, "END Visu3D\n");
  
  fflush(fp);
  fclose(fp);
}



Model *model_load_a(char *filename)
{
  FILE *fp;
  char *string, *tmp;
  int t, cn;

  Point *point, *pv[3];
  Model *model;

  Tri *new_tri;

  fp=fopen(filename, "r");
  
  string=get_line_from_file(fp);
  
  if(strcmp(string, "Visu3D ASCII")) return NULL;
  free(string);

  model=model_new();

  string=get_line_from_file(fp);
	     
  while(string[0]!='\0') {      
    if(strncmp(string, "POINT ", 6)) break;

    free(string);

    point=(Point *)malloc(sizeof(Point));
    for(cn=0;cn<3;cn++) {
      string=get_line_from_file(fp);
      point->v[cn]=atof(string+1)+50;
      free(string);
    }
    model->points=g_list_append(model->points, (gpointer)point);
    
    string=get_line_from_file(fp);
  }

  while(string[0]!='\0') {
    if(strncmp(string, "TRIANGLE ", 9)) break;
    free(string);
    
    string=get_line_from_file(fp);
    if(strcmp(string, " POINTS")) break;
    free(string);

    for(cn=0;cn<3;cn++) {
      string=get_line_from_file(fp);
      tmp=string+2;
      t=atoi(tmp);
      pv[cn]=(Point *)(g_list_nth(model->points, t)->data);
      pv[cn]->order=t;
      free(string);
    }

    string=get_line_from_file(fp);
    if(strcmp(string, " COULEURS")) break;
    free(string);

    string=get_line_from_file(fp);
    tmp=string+2;
    t=atoi(tmp);
    color_rouge=t;
    free(string);

    string=get_line_from_file(fp);
    tmp=string+2;
    t=atoi(tmp);
    color_vert=t;
    free(string);

    string=get_line_from_file(fp);
    tmp=string+2;
    t=atoi(tmp);
    color_bleu=t;
    free(string);

    string=get_line_from_file(fp);
    new_tri=model_add_triangle(model, pv);
  }

  if(strcmp(string, "END Visu3D")) printf("Manque la fin de la signature.\n");
  free(string);
  return model;
}

Model *model_new(void)
{
  Model *m;
  m=(Model *)malloc(sizeof(Model));
  m->points=NULL;
  m->triangles=NULL;
  m->lines=NULL;

  m->list=0;
  m->dirty=1;

  return m;
}

Tri *model_add_triangle(Model *m, Point **p)
{
  Tri *mt;
  int c;

  mt=(Tri *)malloc(sizeof(Tri));

  for(c=0;c<3;c++) {
    if(!g_list_find(m->points, (gpointer)p[c]))
      m->points=g_list_append(m->points, (gpointer)p[c]);
    mt->verticies[c]=p[c];
  }

  mt->rouge=color_rouge;
  mt->vert=color_vert;
  mt->bleu=color_bleu;


  m->triangles=g_list_append(m->triangles, (gpointer)mt);

  return mt;
}

