#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <gtk/gtk.h>
#include <toolbar.h>
#include <fileops.h>

GtkWidget *popup;
GdkPixmap *pixmap_ar[16]; 
GdkBitmap *bitmap_ar[16];
GtkWidget *modal_pixmap[4];

int menu_callback(GtkWidget *widget, gpointer data)
{
  printf("Menu callback: %s\n", (char *)data);
  return 0;
}

void my_quit(GtkWidget *widget, gpointer data)
{
  gtk_exit(0);
}

void toolbar_callback(GtkWidget *widget, GdkEventButton *event, gpointer data)
{
  click_mode[event->button]=(int)data;

  gtk_pixmap_set(GTK_PIXMAP(modal_pixmap[event->button]),
		 pixmap_ar[(int)data], bitmap_ar[(int)data]);
}

GtkWidget *tb_pixmap_new(char *filename, GtkWidget *window, int cm)
{
  GdkPixmap *gdk_pixmap;
  GdkBitmap *mask;
  GtkWidget *pixmap;
  char *real_filename;

  real_filename=malloc(strlen(filename)+strlen(PIXMAP_DIR)+1);
  strcpy(real_filename, PIXMAP_DIR);
  strcat(real_filename, filename);

  gdk_pixmap=gdk_pixmap_create_from_xpm(window->window, &mask,
					&window->style->bg[GTK_STATE_NORMAL],
					real_filename);
  pixmap_ar[cm]=gdk_pixmap;
  bitmap_ar[cm]=mask;
  pixmap=gtk_pixmap_new(gdk_pixmap, mask);
  gtk_widget_show(pixmap);

  free(real_filename);

  return pixmap;
}

GtkWidget *pixmap_new(char *filename, GtkWidget *window)
{
  GdkPixmap *gdk_pixmap;
  GdkBitmap *mask;
  GtkWidget *pixmap;
  char *real_filename;

  real_filename=malloc(strlen(filename)+strlen(PIXMAP_DIR)+1);
  strcpy(real_filename, PIXMAP_DIR);
  strcat(real_filename, filename);

  gdk_pixmap=gdk_pixmap_create_from_xpm(window->window, &mask,
					&window->style->bg[GTK_STATE_NORMAL],
					real_filename);
  pixmap=gtk_pixmap_new(gdk_pixmap, mask);
  gtk_widget_show(pixmap);

  free(real_filename);

  return pixmap;
}

GtkWidget *tb_get_right_half(void)
{
  GtkWidget *hbox, *frame;

  hbox=gtk_hbox_new(FALSE, 0);
  gtk_widget_show(hbox);
  
  frame=gtk_frame_new(NULL);
  gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_OUT);
  gtk_widget_show(frame);
  gtk_container_add(GTK_CONTAINER(frame), hbox);

  modal_pixmap[1]=gtk_pixmap_new(pixmap_ar[click_mode[1]],
				 bitmap_ar[click_mode[1]]);
  gtk_widget_show(modal_pixmap[1]);
  gtk_box_pack_start(GTK_BOX(hbox), modal_pixmap[1], FALSE, FALSE, 2);
  modal_pixmap[2]=gtk_pixmap_new(pixmap_ar[click_mode[2]],
				 bitmap_ar[click_mode[2]]);
  gtk_widget_show(modal_pixmap[2]);
  gtk_box_pack_start(GTK_BOX(hbox), modal_pixmap[2], FALSE, FALSE, 2);
  modal_pixmap[3]=gtk_pixmap_new(pixmap_ar[click_mode[3]],
				 bitmap_ar[click_mode[3]]);
  gtk_widget_show(modal_pixmap[3]);
  gtk_box_pack_start(GTK_BOX(hbox), modal_pixmap[3], FALSE, FALSE, 2);
  
  return frame;
}

GtkWidget *create_toolbar(void)
{
  GtkWidget *toolbar, *frame, *hbox;
  GtkWidget *pixmap;
  GtkWidget *w;

  toolbar = gtk_toolbar_new(GTK_ORIENTATION_HORIZONTAL, GTK_TOOLBAR_BOTH);
  gtk_toolbar_set_style(GTK_TOOLBAR(toolbar), GTK_TOOLBAR_TEXT);
  gtk_toolbar_set_button_relief(GTK_TOOLBAR(toolbar), GTK_RELIEF_NONE);
  gtk_toolbar_set_space_style(GTK_TOOLBAR(toolbar), GTK_TOOLBAR_SPACE_LINE);
  
  gtk_toolbar_append_item(GTK_TOOLBAR(toolbar), NULL, "Quitter",NULL,
			    pixmap_new("exit.xpm", window),
			    GTK_SIGNAL_FUNC(my_quit),
			    (gpointer)popup);
  gtk_toolbar_append_item(GTK_TOOLBAR(toolbar), NULL, "Ouvrir",NULL,
			  pixmap_new("file.xpm", window),
			  GTK_SIGNAL_FUNC(show_load_dialog),
			  (gpointer)popup);
  gtk_toolbar_append_item(GTK_TOOLBAR(toolbar), NULL, "Nouveau",NULL,
			  pixmap_new("new.xpm", window),
			  GTK_SIGNAL_FUNC(show_new_dialog),
			  (gpointer)popup);
  gtk_toolbar_append_item(GTK_TOOLBAR(toolbar), NULL, "Enregistrer",NULL,
			  pixmap_new("save.xpm", window),
			  GTK_SIGNAL_FUNC(save_menu_cb),
			  (gpointer)popup);
  gtk_toolbar_append_space(GTK_TOOLBAR(toolbar));

  pixmap=tb_pixmap_new("pointsel.xpm", window, CM_POINT_SEL);
  w=gtk_toolbar_append_item(GTK_TOOLBAR(toolbar), NULL, "Selection Point",NULL,
			    pixmap,
			    NULL, NULL);
  gtk_signal_connect(GTK_OBJECT(w), "button_press_event",
		     GTK_SIGNAL_FUNC(toolbar_callback), 
		     (gpointer)CM_POINT_SEL);
  pixmap=tb_pixmap_new("pointadd.xpm", window, CM_POINT_ADD);
  w=gtk_toolbar_append_item(GTK_TOOLBAR(toolbar), NULL, "Ajout Point", NULL, 
			    pixmap,
			    NULL, NULL);
  gtk_signal_connect(GTK_OBJECT(w), "button_press_event",
		     GTK_SIGNAL_FUNC(toolbar_callback), 
		     (gpointer)CM_POINT_ADD);
  pixmap=tb_pixmap_new("pointsub.xpm", window, CM_POINT_DEL);
  w=gtk_toolbar_append_item(GTK_TOOLBAR(toolbar), NULL, "Effacement Point",NULL,
			    pixmap,
			    NULL,NULL);
  gtk_signal_connect(GTK_OBJECT(w), "button_press_event",
		     GTK_SIGNAL_FUNC(toolbar_callback), 
		     (gpointer)CM_POINT_DEL);
  
  gtk_toolbar_append_space(GTK_TOOLBAR(toolbar));

  pixmap=tb_pixmap_new("linesel.xpm", window, CM_LINE_SEL);
  w=gtk_toolbar_append_item(GTK_TOOLBAR(toolbar), NULL, "Selection Ligne", NULL, 
			    pixmap,
			    NULL,NULL);
  gtk_signal_connect(GTK_OBJECT(w), "button_press_event",
		     GTK_SIGNAL_FUNC(toolbar_callback), 
		     (gpointer)CM_LINE_SEL);
  pixmap=tb_pixmap_new("lineadd.xpm", window, CM_LINE_ADD);
  w=gtk_toolbar_append_item(GTK_TOOLBAR(toolbar), NULL, "Ajout Ligne", NULL, 
			    pixmap,
			    NULL,NULL);
  gtk_signal_connect(GTK_OBJECT(w), "button_press_event",
		     GTK_SIGNAL_FUNC(toolbar_callback), 
		     (gpointer)CM_LINE_ADD);
  pixmap=tb_pixmap_new("linesub.xpm", window, CM_LINE_DEL);
  w=gtk_toolbar_append_item(GTK_TOOLBAR(toolbar), NULL, "Effacement Ligne", NULL, 
			    pixmap,
			    NULL,NULL);
  gtk_signal_connect(GTK_OBJECT(w), "button_press_event",
		     GTK_SIGNAL_FUNC(toolbar_callback), 
		     (gpointer)CM_LINE_DEL);

  gtk_toolbar_append_space(GTK_TOOLBAR(toolbar));

  pixmap=tb_pixmap_new("trisel.xpm", window, CM_TRI_SEL);
  w=gtk_toolbar_append_item(GTK_TOOLBAR(toolbar), NULL, "Selection Triangle", 
			    NULL, pixmap,
			    NULL,NULL);
  gtk_signal_connect(GTK_OBJECT(w), "button_press_event",
		     GTK_SIGNAL_FUNC(toolbar_callback), 
		     (gpointer)CM_TRI_SEL);
  pixmap=tb_pixmap_new("triadd.xpm", window, CM_TRI_ADD);
  w=gtk_toolbar_append_item(GTK_TOOLBAR(toolbar), NULL, "Ajout Triangle", NULL, 
			    pixmap,
			    NULL,NULL);
  gtk_signal_connect(GTK_OBJECT(w), "button_press_event",
		     GTK_SIGNAL_FUNC(toolbar_callback), 
		     (gpointer)CM_TRI_ADD);
  pixmap=tb_pixmap_new("trisub.xpm", window, CM_TRI_DEL);
  w=gtk_toolbar_append_item(GTK_TOOLBAR(toolbar), NULL, "Effacement Triangle", 
			    NULL, pixmap,
			    NULL,NULL);
  gtk_signal_connect(GTK_OBJECT(w), "button_press_event",
		     GTK_SIGNAL_FUNC(toolbar_callback), 
		     (gpointer)CM_TRI_DEL);
  
  gtk_widget_show(toolbar);
  
  frame=gtk_frame_new(NULL);
  gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_OUT);
  gtk_widget_show(frame);
  gtk_container_add(GTK_CONTAINER(frame), toolbar);
  hbox=gtk_hbox_new(FALSE, 0);
  gtk_box_pack_start(GTK_BOX(hbox), frame, TRUE, TRUE, 0);

  gtk_box_pack_start(GTK_BOX(hbox), tb_get_right_half(),
		     FALSE, FALSE, 0);
  gtk_widget_show(hbox);

  return hbox;
}
  
  
